@@ -1,13 +1,15 @@
 """An implementation of the Wasserstein metric between two compositional vectors.
+Aka Earth Mover's distance, an appropriate metric for comparing two PDDs.
 
-Copyright (C) 2020 Cameron Hargreaves. This code is part of the Element Movers 
+Copyright (C) 2020 Cameron Hargreaves. This code is part of the Element Movers
 Distance project https://github.com/lrcfmd/ElMD.
 """
 
+import numba
 import numpy as np
-from numba import njit
 
-@njit(cache=True)
+
+@numba.njit(cache=True)
 def network_simplex(source_demands, sink_demands, network_costs):
     """
     This is a port of the network simplex algorithm implented by Loïc Séguin-C
@@ -25,18 +27,15 @@ def network_simplex(source_demands, sink_demands, network_costs):
            optimization.
            INFOR 17(1):16--34. 1979.
     """
-    
+
+    n_sources, n_sinks = source_demands.shape[0], sink_demands.shape[0]
     network_costs = network_costs.ravel()
-    
+
     # Constant used throughout for conversions from floating point to integer
     fp_multiplier = np.array([1000000], dtype=np.int64)
 
-    # Using numerical ordering is nice for indexing
-    sources = np.arange(source_demands.shape[0]).astype(np.int64)
-    sinks = np.arange(sink_demands.shape[0]).astype(np.int64) + source_demands.shape[0]
-
     # Add one additional node for a dummy source and sink
-    nodes = np.arange(source_demands.shape[0] + sink_demands.shape[0]).astype(np.int64)
+    nodes = np.arange(n_sources + n_sinks).astype(np.int64)
 
     # Multiply by a large number and cast to int to remove floating points
     source_d_fp = source_demands * fp_multiplier.astype(np.int64)
@@ -47,11 +46,10 @@ def network_simplex(source_demands, sink_demands, network_costs):
     # FP conversion error correction
     source_sum = np.sum(source_d_int)
     sink_sum = np.sum(sink_d_int)
-    
+
     if  source_sum < sink_sum:
         source_ind = np.argmax(source_d_int)
         source_d_int[source_ind] += sink_sum - source_sum
-
     elif sink_sum < source_sum:
         sink_ind = np.argmax(sink_d_int)
         sink_d_int[sink_ind] += source_sum - sink_sum
@@ -60,8 +58,10 @@ def network_simplex(source_demands, sink_demands, network_costs):
     demands = np.concatenate((-source_d_int, sink_d_int)).astype(np.int64)
 
     # Create fully connected arcs between all sources and sinks
-    conn_tails = np.array([i for i in range(len(sources)) for _ in range(len(sinks))], dtype=np.int64)
-    conn_heads = np.array([j + sources.shape[0] for _ in range(len(sources)) for j in range(len(sinks))], dtype=np.int64)
+    conn_tails = np.array([i for i in range(n_sources) for _ in range(n_sinks)],
+                          dtype=np.int64)
+    conn_heads = np.array([j + n_sources for _ in range(n_sources) for j in range(n_sinks)],
+                          dtype=np.int64)
 
     # Add arcs to and from the dummy node
     dummy_tails = []
@@ -81,7 +81,10 @@ def network_simplex(source_demands, sink_demands, network_costs):
 
     # Create costs and capacities for the arcs between nodes
     network_costs = network_costs * fp_multiplier
-    network_capac = np.array([np.array([source_demands[i], sink_demands[j]]).min() for i in range(len(sources)) for j in range(len(sinks))], dtype=np.float64) * fp_multiplier
+    network_capac = np.array([np.array([source_demands[i], sink_demands[j]]).min()
+                              for i in range(n_sources)
+                              for j in range(n_sinks)],
+                             dtype=np.float64) * fp_multiplier
 
     # TODO finish?
     # If there is only one node on either side we can return capacity and costs
@@ -92,7 +95,10 @@ def network_simplex(source_demands, sink_demands, network_costs):
     # inf_arr = (np.sum(network_capac.astype(np.int64)), np.sum(np.absolute(network_costs)), np.max(np.absolute(demands)))
 
     # Set a suitably high integer for infinity
-    faux_inf = 3 * np.max(np.array((np.sum(network_capac.astype(np.int64)), np.sum(np.absolute(network_costs)), np.max(np.absolute(demands))), dtype=np.int64))
+    faux_inf = 3 * np.max(np.array((np.sum(network_capac.astype(np.int64)),
+                                    np.sum(np.absolute(network_costs)),
+                                    np.max(np.absolute(demands))),
+                                   dtype=np.int64))
 
     # Add the costs and capacities to the dummy nodes
     costs = np.concatenate((network_costs, np.ones(nodes.shape[0]) * faux_inf)).astype(np.int64)
@@ -108,18 +114,17 @@ def network_simplex(source_demands, sink_demands, network_costs):
     # General arrays for the spanning tree
     potentials = np.array([faux_inf if d <= 0 else -faux_inf for d in demands]).T
     parent = np.concatenate((np.ones(n) * -1, np.array([-2]))).astype(np.int64)
-    edge = np.arange(e, e+n).astype(np.int64)
+    edge = np.arange(e, e + n).astype(np.int64)
     size = np.concatenate((np.ones(n), np.array([n + 1]))).astype(np.int64)
-    next = np.concatenate((np.arange(1, n), np.array([-1, 0]))).astype(np.int64)
-    prev = np.arange(-1, n)          # previous nodes in depth-first thread
-    last = np.concatenate((np.arange(n), np.array([n - 1]))).astype(np.int64)     # last descendants in depth-first thread
+    next_node = np.concatenate((np.arange(1, n), np.array([-1, 0]))).astype(np.int64)
+    prev_node = np.arange(-1, n)          # previous nodes in depth-first thread
+    last_node = np.concatenate((np.arange(n), np.array([n - 1]))).astype(np.int64)  # last descendants in depth-first thread
+    f = 0
 
     ###########################################################################
     # Main Pivot loop
     ###########################################################################
 
-    f = 0
-
     while True:
         i, p, q, f = find_entering_edges(e, f, tails, heads, costs, potentials, flows)
         if p == -1: # If no entering edges then the optimal score is found
@@ -129,8 +134,7 @@ def network_simplex(source_demands, sink_demands, network_costs):
         j, s, t = find_leaving_edge(cycle_nodes, cycle_edges, capac, flows, tails, heads)
         augment_flow(cycle_nodes, cycle_edges, residual_capacity(j, s, capac, flows, tails), tails, flows)
 
-        if i != j:  # Do nothing more if the entering edge is the same as the
-                    # the leaving edge.
+        if i != j:  # Do nothing more if the entering edge is the same as the leaving edge.
             if parent[t] != s:
                 # Ensure that s is the parent of t.
                 s, t = t, s
@@ -139,11 +143,14 @@ def network_simplex(source_demands, sink_demands, network_costs):
                 # Ensure that q is in the subtree rooted at t.
                 p, q = q, p
 
-            remove_edge(s, t, size, prev, last, next, parent, edge)
-            make_root(q, parent, size, last, prev, next, edge)
-            add_edge(i, p, q, next, prev, last, size, parent, edge)
-            update_potentials(i, p, q, heads, potentials, costs, last, next)
+            remove_edge(s, t, size, prev_node, last_node, next_node, parent, edge)
+            make_root(q, parent, size, last_node, prev_node, next_node, edge)
+            add_edge(i, p, q, next_node, prev_node, last_node, size, parent, edge)
+            update_potentials(i, p, q, heads, potentials, costs, last_node, next_node)
 
+    # final_emd, final_flows = emd_from_flows_and_costs(flows[:e], costs[:e], fp_multiplier)
+    # return final_emd, final_flows
+    
     flow_cost = 0
     final_flows = flows[:e].astype(np.float64)
     edge_costs = costs[:e].astype(np.float64)
@@ -152,15 +159,34 @@ def network_simplex(source_demands, sink_demands, network_costs):
     for arc_ind, flow in np.ndenumerate(final_flows):
         flow_cost += flow * edge_costs[arc_ind]
 
-    final = flow_cost / fp_multiplier 
+    final = flow_cost / fp_multiplier
     final = final.astype(np.float64)
-    final = final / fp_multiplier 
+    final = final / fp_multiplier
 
     final_flows = final_flows / fp_multiplier
 
     return final[0], final_flows
 
-@njit(cache=True)
+
+# @numba.njit(cache=True)
+# def emd_from_flows_and_costs(flows, costs, fp_multiplier):
+#     """Calculate final EMD from flow and cost matrices."""
+#     flow_cost = 0
+#     final_flows = flows.astype(np.float64)
+#     edge_costs = costs.astype(np.float64)
+
+#     # dot product is returning wrong values for some reason...
+#     for arc_ind, flow in np.ndenumerate(final_flows):
+#         flow_cost += flow * edge_costs[arc_ind]
+
+#     final = flow_cost / fp_multiplier
+#     final = final.astype(np.float64)
+#     final = final / fp_multiplier
+#     final_flows = final_flows / fp_multiplier
+#     return final[0], final_flows
+
+
+@numba.njit(cache=True)
 def reduced_cost(i, costs, potentials, tails, heads, flows):
     """Return the reduced cost of an edge i.
     """
@@ -171,7 +197,8 @@ def reduced_cost(i, costs, potentials, tails, heads, flows):
     else:
         return -c
 
-@njit(cache=True)
+
+@numba.njit(cache=True)
 def find_entering_edges(e, f, tails, heads, costs, potentials, flows):
     """Yield entering edges until none can be found.
     """
@@ -227,7 +254,8 @@ def find_entering_edges(e, f, tails, heads, costs, potentials, flows):
     # All edges have nonnegative reduced costs. The flow is optimal.
     return -1, -1, -1, -1
 
-@njit(cache=True)
+
+@numba.njit(cache=True)
 def find_apex(p, q, size, parent):
     """Find the lowest common ancestor of nodes p and q in the spanning
     tree.
@@ -251,7 +279,8 @@ def find_apex(p, q, size, parent):
             else:
                 return p
 
-@njit(cache=True)
+
+@numba.njit(cache=True)
 def trace_path(p, w, edge, parent):
     """Return the nodes and edges on the path from node p to its ancestor
     w.
@@ -266,7 +295,8 @@ def trace_path(p, w, edge, parent):
 
     return cycle_nodes, cycle_edges
 
-@njit(cache=True)
+
+@numba.njit(cache=True)
 def find_cycle(i, p, q, size, edge, parent):
     """Return the nodes and edges on the cycle containing edge i == (p, q)
     when the latter is added to the spanning tree.
@@ -290,7 +320,8 @@ def find_cycle(i, p, q, size, edge, parent):
 
     return cycle_nodes, cycle_edges
 
-@njit(cache=True)
+
+@numba.njit(cache=True)
 def residual_capacity(i, p, capac, flows, tails):
     """Return the residual capacity of an edge i in the direction away
     from its endpoint p.
@@ -301,7 +332,8 @@ def residual_capacity(i, p, capac, flows, tails):
     else:
         return flows[np.int64(i)]
 
-@njit(cache=True)
+
+@numba.njit(cache=True)
 def find_leaving_edge(cycle_nodes, cycle_edges, capac, flows, tails, heads):
     """Return the leaving edge in a cycle represented by cycle_nodes and
     cycle_edges.
@@ -323,45 +355,48 @@ def find_leaving_edge(cycle_nodes, cycle_edges, capac, flows, tails, heads):
     t = heads[np.int64(j)] if tails[np.int64(j)] == s else tails[np.int64(j)]
     return j, s, t
 
-@njit(cache=True)
+
+@numba.njit(cache=True)
 def augment_flow(cycle_nodes, cycle_edges, f, tails, flows):
     """Augment f units of flow along a cycle representing Wn with cycle_edges.
     """
     for i, p in zip(cycle_edges, cycle_nodes):
-        if tails[int(i)] == np.int64(p):
-            flows[int(i)] += f
+        if tails[np.int64(i)] == np.int64(p):
+            flows[np.int64(i)] += f
         else:
-            flows[int(i)] -= f
+            flows[np.int64(i)] -= f
 
-@njit(cache=True)
-def trace_subtree(p, last, next):
+
+@numba.njit(cache=True)
+def trace_subtree(p, last_node, next_node):
     """Yield the nodes in the subtree rooted at a node p.
     """
     tree = []
     tree.append(p)
 
-    l = last[p]
+    l = last_node[p]
     while p != l:
-        p = next[p]
+        p = next_node[p]
         tree.append(p)
 
     return np.array(tree, dtype=np.int64)
 
-@njit(cache=True)
-def remove_edge(s, t, size, prev, last, next, parent, edge):
+
+@numba.njit(cache=True)
+def remove_edge(s, t, size, prev, last, next_node, parent, edge):
     """Remove an edge (s, t) where parent[t] == s from the spanning tree.
     """
     size_t = size[t]
     prev_t = prev[t]
     last_t = last[t]
-    next_last_t = next[last_t]
+    next_last_t = next_node[last_t]
     # Remove (s, t).
     parent[t] = -2
     edge[t] = -2
     # Remove the subtree rooted at t from the depth-first thread.
-    next[prev_t] = next_last_t
+    next_node[prev_t] = next_last_t
     prev[next_last_t] = prev_t
-    next[last_t] = t
+    next_node[last_t] = t
     prev[t] = last_t
 
     # Update the subtree sizes and last descendants of the (old) ancestors
@@ -372,8 +407,9 @@ def remove_edge(s, t, size, prev, last, next, parent, edge):
             last[s] = prev_t
         s = parent[s]
 
-@njit(cache=True)
-def make_root(q, parent, size, last, prev, next, edge):
+
+@numba.njit(cache=True)
+def make_root(q, parent, size, last, prev, next_node, edge):
     """
     Make a node q the root of its containing subtree.
     """
@@ -392,7 +428,7 @@ def make_root(q, parent, size, last, prev, next, edge):
         last_p = last[p]
         prev_q = prev[q]
         last_q = last[q]
-        next_last_q = next[last_q]
+        next_last_q = next_node[last_q]
 
         # Make p a child of q.
         parent[p] = q
@@ -403,9 +439,9 @@ def make_root(q, parent, size, last, prev, next, edge):
         size[q] = size_p
 
         # Remove the subtree rooted at q from the depth-first thread.
-        next[prev_q] = next_last_q
+        next_node[prev_q] = next_last_q
         prev[next_last_q] = prev_q
-        next[last_q] = q
+        next_node[last_q] = q
         prev[q] = last_q
 
         if last_p == last_q:
@@ -415,28 +451,29 @@ def make_root(q, parent, size, last, prev, next, edge):
         # Add the remaining parts of the subtree rooted at p as a subtree
         # of q in the depth-first thread.
         prev[p] = last_q
-        next[last_q] = p
-        next[last_p] = q
+        next_node[last_q] = p
+        next_node[last_p] = q
         prev[q] = last_p
         last[q] = last_p
 
-@njit(cache=True)
-def add_edge(i, p, q, next, prev, last, size, parent, edge):
+
+@numba.njit(cache=True)
+def add_edge(i, p, q, next_node, prev_node, last, size, parent, edge):
     """Add an edge (p, q) to the spanning tree where q is the root of a
     subtree.
     """
     last_p = last[p]
-    next_last_p = next[last_p]
+    next_last_p = next_node[last_p]
     size_q = size[q]
     last_q = last[q]
     # Make q a child of p.
     parent[q] = p
     edge[q] = i
     # Insert the subtree rooted at q into the depth-first thread.
-    next[last_p] = q
-    prev[q] = last_p
-    prev[next_last_p] = last_q
-    next[last_q] = next_last_p
+    next_node[last_p] = q
+    prev_node[q] = last_p
+    prev_node[next_last_p] = last_q
+    next_node[last_q] = next_last_p
 
     # Update the subtree sizes and last descendants of the (new) ancestors
     # of q.
@@ -446,8 +483,9 @@ def add_edge(i, p, q, next, prev, last, size, parent, edge):
             last[p] = last_q
         p = parent[p]
 
-@njit(cache=True)
-def update_potentials(i, p, q, heads, potentials, costs, last, next):
+
+@numba.njit(cache=True)
+def update_potentials(i, p, q, heads, potentials, costs, last_node, next_node):
     """Update the potentials of the nodes in the subtree rooted at a node
     q connected to its parent p by an edge i.
     """
@@ -456,6 +494,6 @@ def update_potentials(i, p, q, heads, potentials, costs, last, next):
     else:
         d = potentials[p] + costs[i] - potentials[q]
 
-    tree = trace_subtree(q, last, next)
+    tree = trace_subtree(q, last_node, next_node)
     for q in tree:
-        potentials[q] += d
\ No newline at end of file
+        potentials[q] += d
