@@ -1,50 +1,50 @@
-"""Implements the class :class:`PeriodicSet` representing a periodic set, 
+"""Implements the class :class:`PeriodicSet` representing a periodic set,
 defined by a motif and unit cell.
 
-This is the object type yielded by the readers :class:`.io.CifReader` and 
-:class:`.io.CSDReader`. The :class:`PeriodicSet` can be passed as the first argument 
+This is the object type yielded by the readers :class:`.io.CifReader` and
+:class:`.io.CSDReader`. The :class:`PeriodicSet` can be passed as the first argument
 to :func:`.calculate.AMD` or :func:`.calculate.PDD` to calculate its invariants.
-They can be written to a file with :class:`.io.SetWriter` which can be read with 
+They can be written to a file with :class:`.io.SetWriter` which can be read with
 :class:`.io.SetReader`.
 """
 
 from typing import Optional
 import numpy as np
 
+
 class PeriodicSet:
-    """Represents a periodic set (which mathematically represents a crystal). 
-    
-    Has attributes motif, cell and name (which can be :const:`None`). 
+    """A periodic set is the mathematical representation of a crystal by putting a
+    single point in the center of every atom. A periodic set is defined by a basis
+    (unit cell) and collection of points (motif) which repeats according to the basis.
+    Has attributes motif, cell and name (which can be :const:`None`).
+
     :class:`PeriodicSet` objects are returned by the readers in the :mod:`.io` module.
+    Instances of this object can be passed to :func:`.calculate.AMD` or
+    :func:`.calculate.PDD`.
     """
 
-    def __init__(self, motif: np.ndarray, 
-                       cell: np.ndarray, 
-                       name: Optional[str] = None, 
-                       **kwargs):
-        
+    def __init__(
+            self,
+            motif: np.ndarray,
+            cell: np.ndarray,
+            name: Optional[str] = None,
+            **kwargs):
+
         self.motif = motif
-        self.cell  = cell
-        self.name  = name
+        self.cell = cell
+        self.name = name
         self.tags = kwargs
 
-    def __getattr__(self, attr):
-        if 'tags' not in self.__dict__:
-            self.tags = {}
-        if attr in self.tags:
-            return self.tags[attr]
-        else:
-            raise AttributeError(f"{self.__class__.__name__} object has no attribute or tag {attr}")
-
     def __str__(self):
+
         m, dims = self.motif.shape
         return f"PeriodicSet({self.name}: {m} motif points in {dims} dimensions)"
-    
+
     def __repr__(self):
         return f"PeriodicSet(name: {self.name}, cell: {self.cell}, motif: {self.motif}, tags: {self.tags})"
-    
+
     # used for debugging, checks if the motif/cell agree point for point
-    # (disregarding order), NOT up to isometry. 
+    # (disregarding order), NOT up to isometry.
     def __eq__(self, other):
 
         if self.cell.shape != other.cell.shape or self.motif.shape != other.motif.shape:
@@ -58,7 +58,7 @@ class PeriodicSet:
         diffs = np.amax(np.abs(other.motif[:, None] - self.motif), axis=-1)
         if not np.all((np.amin(diffs, axis=0) <= 1e-6) & (np.amin(diffs, axis=-1) <= 1e-6)):
             return False
-        
+
         shared_tags = set(self.tags.keys()).intersection(set(other.tags.keys()))
         for tag in shared_tags:
             if np.isscalar(self.tags[tag]):
@@ -73,34 +73,28 @@ class PeriodicSet:
                         return False
 
         return True
-    
+
+    def __getattr__(self, attr):
+
+        if 'tags' not in self.__dict__:
+            self.tags = {}
+        if attr in self.tags:
+            return self.tags[attr]
+
+        raise AttributeError(f"{self.__class__.__name__} object has no attribute or tag {attr}")
+
     def __ne__(self, other):
         return not self.__eq__(other)
-    
+
+    def copy(self):
+        """Return copy of the PeriodicSet."""
+        return PeriodicSet(self.motif, self.cell, name=self.name, **self.tags)
+
     def astype(self, dtype):
-        """Returns copy of the :class:`PeriodicSet` with ``.motif`` 
+        """Returns copy of the :class:`PeriodicSet` with ``.motif``
         and ``.cell`` casted to ``dtype``."""
-        
-        return PeriodicSet(self.motif.astype(dtype), 
+
+        return PeriodicSet(self.motif.astype(dtype),
                            self.cell.astype(dtype),
                            name=self.name,
                            **self.tags)
-
-    def to_dict(self) -> dict:
-        """Return ``dict`` with scalar data in the tags of the :class:`PeriodicSet`.
-        
-        Format of returned ``dict`` is for example::
-        
-            {
-                'density': 1.231, 
-                'family': 'CBMZPN', 
-                ...
-            }
-        """
-        
-        data = {}
-        for tag in self.tags:
-            if np.isscalar(self.tags[tag]):
-                data[tag] = self.tags[tag]
-
-        return data
\ No newline at end of file
